@@ -1,28 +1,28 @@
+import freeOrionAIInterface as fo
 import math
-from logging import error, warning, debug
-
-import freeOrionAIInterface as fo  # pylint: disable=import-error
+from logging import debug, error, warning
+from typing import List, Sequence, Set, Tuple, Union
 
 import AIDependencies
-import CombatRatingsAI
 import MoveUtilsAI
 from AIDependencies import INVALID_ID
 from aistate_interface import get_aistate
-from freeorion_tools import assertion_fails
+from CombatRatingsAI import get_fleet_rating, get_ship_combat_stats, rating_needed
+from common.fo_typing import PlanetId
 from EnumsAI import MissionType, ShipRoleType
+from freeorion_tools import assertion_fails, combine_ratings
 from ShipDesignAI import get_ship_part
-from target import TargetPlanet, TargetFleet, TargetSystem
+from target import TargetFleet, TargetPlanet, TargetSystem
+from universe.system_network import get_neighbors, get_shortest_distance
 
 
-def stats_meet_reqs(stats, requirements):
-    """Check if (fleet) stats meet requirements.
+def stats_meet_reqs(stats: dict, requirements: dict) -> bool:
+    """
+    Check if (fleet) stats meet requirements.
 
     :param stats: Stats (of fleet)
-    :type stats: dict
     :param requirements: Requirements
-    :type requirements: dict
     :return: True if requirements are met.
-    :rtype: bool
     """
     for key in requirements:
         if key not in stats:  # skip requirements not related to stats
@@ -34,18 +34,18 @@ def stats_meet_reqs(stats, requirements):
     return True
 
 
-def count_troops_in_fleet(fleet_id):
-    """Get the number of troops in the fleet.
+def count_troops_in_fleet(fleet_id: int) -> float:
+    """
+    Get the number of troops in the fleet.
 
     :param fleet_id: fleet to be queried
-    :type fleet_id: int
     :return: total troopCapacity of the fleet
     """
     universe = fo.getUniverse()
     fleet = universe.getFleet(fleet_id)
     if not fleet:
-        return 0
-    fleet_troop_capacity = 0
+        return 0.0
+    fleet_troop_capacity = 0.0
     for ship_id in fleet.shipIDs:
         ship = universe.getShip(ship_id)
         if ship:
@@ -53,15 +53,12 @@ def count_troops_in_fleet(fleet_id):
     return fleet_troop_capacity
 
 
-def get_targeted_planet_ids(planet_ids, mission_type):
+def get_targeted_planet_ids(planet_ids: Sequence[PlanetId], mission_type: MissionType) -> List[PlanetId]:
     """Find the planets that are targets of the specified mission type.
 
     :param planet_ids: planets to be queried
-    :type planet_ids: list[int]
     :param mission_type:
-    :type mission_type: MissionType
     :return: Subset of *planet_ids* targeted by *mission_type*
-    :rtype: list[int]
     """
     selected_fleet_missions = get_aistate().get_fleet_missions_with_any_mission_types([mission_type])
     targeted_planets = []
@@ -76,8 +73,16 @@ def get_targeted_planet_ids(planet_ids, mission_type):
 
 # TODO: Avoid mutable arguments and use return values instead
 # TODO: Use Dijkstra's algorithm instead of BFS to consider starlane length
-def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
-                           fleet_pool_set, fleet_list, species="", ensure_return=False):
+def get_fleets_for_mission(  # noqa: max-complexity
+    target_stats: dict,
+    min_stats: dict,
+    cur_stats: dict,
+    starting_system: int,
+    fleet_pool_set: Set[int],
+    fleet_list: List[int],
+    species: str = "",
+    ensure_return: bool = False,
+) -> List[int]:
     """Get fleets for a mission.
 
     Implements breadth-first search through systems starting at the **starting_sytem**.
@@ -89,22 +94,14 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
     an emergency use of the found fleets in fleet_list; if not to be used they should be added back to the main pool.
 
     :param target_stats: stats the fleet should ideally meet
-    :type target_stats: dict
     :param min_stats: minimum stats the final fleet must meet to be accepted
-    :type min_stats: dict
     :param cur_stats: (**mutated**) stat summary of selected fleets
-    :type cur_stats: dict
     :param starting_system: system_id where breadth-first-search is centered
-    :type starting_system: int
     :param fleet_pool_set: (**mutated**) fleets allowed to be selected. Split fleed_ids are added, used ones removed.
-    :type: fleet_pool_set: set[int]
     :param fleet_list: (**mutated**) fleets that are selected for the mission. Gets filled during the call.
-    :type fleet_list: list[int]
     :param species: species for colonization mission
-    :type species: str
     :param bool ensure_return: If true, fleet must have sufficient fuel to return into supply after mission
     :return: List of selected fleet_ids or empty list if couldn't meet minimum requirements.
-    :rtype: list[int]
     """
     universe = fo.getUniverse()
     colonization_roles = (ShipRoleType.CIVILIAN_COLONISATION, ShipRoleType.BASE_COLONISATION)
@@ -116,7 +113,7 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
         this_system_id = systems_enqueued.pop(0)
         this_system_obj = TargetSystem(this_system_id)
         systems_visited.append(this_system_id)
-        accessible_fleets = aistate.systemStatus.get(this_system_id, {}).get('myFleetsAccessible', [])
+        accessible_fleets = aistate.systemStatus.get(this_system_id, {}).get("myFleetsAccessible", [])
         fleets_here = [fid for fid in accessible_fleets if fid in fleet_pool_set]
         # loop over all fleets in the system, split them if possible and select suitable ships
         while fleets_here:
@@ -132,33 +129,35 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
                 fleet_pool_set.update(new_fleets)
                 fleets_here.extend(new_fleets)
 
-            if ('target_system' in target_stats and
-                    not MoveUtilsAI.can_travel_to_system(fleet_id, this_system_obj,
-                                                         target_stats['target_system'],
-                                                         ensure_return=ensure_return)):
+            if "target_system" in target_stats and not MoveUtilsAI.can_travel_to_system(
+                fleet_id, this_system_obj, target_stats["target_system"], ensure_return=ensure_return
+            ):
                 continue
 
             # check species for colonization missions
             if species:
                 for ship_id in fleet.shipIDs:
                     ship = universe.getShip(ship_id)
-                    if (ship and aistate.get_ship_role(ship.design.id) in colonization_roles and
-                            species == ship.speciesName):
+                    if (
+                        ship
+                        and aistate.get_ship_role(ship.design.id) in colonization_roles
+                        and species == ship.speciesName
+                    ):
                         break
                 else:  # no suitable species found
                     continue
             # check troop capacity for invasion missions
             troop_capacity = 0
-            if 'troopCapacity' in target_stats:
+            if "troopCapacity" in target_stats:
                 troop_capacity = count_troops_in_fleet(fleet_id)
                 if troop_capacity <= 0:
                     continue
 
             # check if we need additional rating vs planets
             this_rating_vs_planets = 0
-            if 'ratingVsPlanets' in target_stats:
+            if "ratingVsPlanets" in target_stats:
                 this_rating_vs_planets = aistate.get_rating(fleet_id, against_planets=True)
-                if this_rating_vs_planets <= 0 and cur_stats.get('rating', 0) >= target_stats.get('rating', 0):
+                if this_rating_vs_planets <= 0 and cur_stats.get("rating", 0) >= target_stats.get("rating", 0):
                     # we already have enough general rating, so do not add any more warships useless against planets
                     continue
 
@@ -171,24 +170,28 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
             fleet_list.append(fleet_id)
 
             this_rating = aistate.get_rating(fleet_id)
-            cur_stats['rating'] = CombatRatingsAI.combine_ratings(cur_stats.get('rating', 0), this_rating)
-            if 'ratingVsPlanets' in target_stats:
-                cur_stats['ratingVsPlanets'] = CombatRatingsAI.combine_ratings(cur_stats.get('ratingVsPlanets', 0),
-                                                                               this_rating_vs_planets)
-            if 'troopCapacity' in target_stats:
-                cur_stats['troopCapacity'] = cur_stats.get('troopCapacity', 0) + troop_capacity
+            cur_stats["rating"] = combine_ratings(cur_stats.get("rating", 0), this_rating)
+            if "ratingVsPlanets" in target_stats:
+                cur_stats["ratingVsPlanets"] = combine_ratings(
+                    cur_stats.get("ratingVsPlanets", 0), this_rating_vs_planets
+                )
+            if "troopCapacity" in target_stats:
+                cur_stats["troopCapacity"] = cur_stats.get("troopCapacity", 0) + troop_capacity
             # if we already meet the requirements, we can stop looking for more ships
-            if (sum(len(universe.getFleet(fid).shipIDs) for fid in fleet_list) >= 1) \
-                    and stats_meet_reqs(cur_stats, target_stats):
+            if (sum(len(universe.getFleet(fid).shipIDs) for fid in fleet_list) >= 1) and stats_meet_reqs(
+                cur_stats, target_stats
+            ):
                 return fleet_list
 
         # finished system without meeting requirements. Add neighboring systems to search queue.
-        for neighbor_id in universe.getImmediateNeighbors(this_system_id, fo.empireID()):
-            if all((
+        for neighbor_id in get_neighbors(this_system_id):
+            if all(
+                (
                     neighbor_id not in systems_visited,
                     neighbor_id not in systems_enqueued,
-                    neighbor_id in aistate.exploredSystemIDs
-            )):
+                    neighbor_id in aistate.exploredSystemIDs,
+                )
+            ):
                 systems_enqueued.append(neighbor_id)
     # we ran out of systems or fleets to check but did not meet requirements yet.
     if stats_meet_reqs(cur_stats, min_stats) and any(universe.getFleet(fid).shipIDs for fid in fleet_list):
@@ -197,13 +200,11 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
         return []
 
 
-def split_fleet(fleet_id):
+def split_fleet(fleet_id: int) -> List[int]:
     """Split a fleet into its ships.
 
     :param fleet_id: fleet to be split.
-    :type fleet_id: int
     :return: New fleets. Empty if couldn't split.
-    :rtype: list[int]
     """
     universe = fo.getUniverse()
     empire_id = fo.empireID()
@@ -231,41 +232,42 @@ def split_fleet(fleet_id):
     return new_fleets
 
 
-def split_ship_from_fleet(fleet_id, ship_id):
+def split_ship_from_fleet(fleet_id, ship_id) -> int:
+    """Try to split a ship from the fleet, creating a new fleet.
+
+    :return: ID of the newly created fleet or INVALID_ID if failed
+    """
     universe = fo.getUniverse()
     fleet = universe.getFleet(fleet_id)
     if assertion_fails(fleet is not None):
-        return
+        return INVALID_ID
 
     if assertion_fails(ship_id in fleet.shipIDs):
-        return
+        return INVALID_ID
 
     if assertion_fails(fleet.numShips > 1, "Can't split last ship from fleet"):
-        return
+        return INVALID_ID
 
     new_fleet_id = fo.issueNewFleetOrder("Fleet %4d" % ship_id, ship_id)
-    if new_fleet_id:
+    if new_fleet_id != INVALID_ID:
         aistate = get_aistate()
         new_fleet = universe.getFleet(new_fleet_id)
         if not new_fleet:
             warning("Newly split fleet %d not available from universe" % new_fleet_id)
-        debug("Successfully split ship %d from fleet %d into new fleet %d",
-              ship_id, fleet_id, new_fleet_id)
+        debug("Successfully split ship %d from fleet %d into new fleet %d", ship_id, fleet_id, new_fleet_id)
         fo.issueRenameOrder(new_fleet_id, "Fleet %4d" % new_fleet_id)  # to ease review of debugging logs
         fo.issueAggressionOrder(new_fleet_id, True)
         aistate.update_fleet_rating(new_fleet_id)
         aistate.newlySplitFleets[new_fleet_id] = True
         # register the new fleets so AI logic is aware of them
         sys_status = aistate.systemStatus.setdefault(fleet.systemID, {})
-        sys_status['myfleets'].append(new_fleet_id)
-        sys_status['myFleetsAccessible'].append(new_fleet_id)
+        sys_status["myfleets"].append(new_fleet_id)
+        sys_status["myFleetsAccessible"].append(new_fleet_id)
     else:
         if fleet.systemID == INVALID_ID:
-            warning("Tried to split ship id (%d) from fleet %d when fleet is in starlane" % (
-                ship_id, fleet_id))
+            warning("Tried to split ship id (%d) from fleet %d when fleet is in starlane" % (ship_id, fleet_id))
         else:
-            warning("Got no fleet ID back after trying to split ship id (%d) from fleet %d" % (
-                ship_id, fleet_id))
+            warning("Got no fleet ID back after trying to split ship id (%d) from fleet %d" % (ship_id, fleet_id))
     return new_fleet_id
 
 
@@ -281,22 +283,24 @@ def merge_fleet_a_into_b(fleet_a_id, fleet_b_id, leave_rating=0, need_rating=0,
         return 0
 
     # TODO: Should this rate against specific enemy?
-    remaining_rating = CombatRatingsAI.get_fleet_rating(fleet_a_id)
+    remaining_rating = get_fleet_rating(fleet_a_id)
     transferred_rating = 0
     for ship_id in fleet_a.shipIDs:
         this_ship = universe.getShip(ship_id)
         if not this_ship:
             continue
-        this_rating = CombatRatingsAI.ShipCombatStats(ship_id).get_rating()
-        remaining_rating = CombatRatingsAI.rating_needed(remaining_rating, this_rating)
+        this_rating = get_ship_combat_stats(ship_id).get_rating()
+        remaining_rating = rating_needed(remaining_rating, this_rating)
         if remaining_rating < leave_rating:  # merging this would leave old fleet under minimum rating, try other ships.
             continue
         transferred = fo.issueFleetTransferOrder(ship_id, fleet_b_id)
         if transferred:
-            transferred_rating = CombatRatingsAI.combine_ratings(transferred_rating, this_rating)
+            transferred_rating = combine_ratings(transferred_rating, this_rating)
         else:
-            debug("  *** transfer of ship %4d, formerly of fleet %4d, into fleet %4d failed; %s" % (
-                ship_id, fleet_a_id, fleet_b_id, (" context is %s" % context) if context else ""))
+            debug(
+                "  *** transfer of ship %4d, formerly of fleet %4d, into fleet %4d failed; %s"
+                % (ship_id, fleet_a_id, fleet_b_id, (" context is %s" % context) if context else "")
+            )
         if need_rating != 0 and need_rating <= transferred_rating:
             break
     fleet_a = universe.getFleet(fleet_a_id)
@@ -372,7 +376,7 @@ def extract_fleet_ids_without_mission_types(fleets_ids):
     return [fleet_id for fleet_id in fleets_ids if not aistate.get_fleet_mission(fleet_id).type]
 
 
-def assess_fleet_role(fleet_id):
+def assess_fleet_role(fleet_id):  # noqa: max-complexity
     """
     Assesses ShipRoles represented in a fleet and
     returns a corresponding overall fleetRole (of type MissionType).
@@ -428,7 +432,7 @@ def assess_fleet_role(fleet_id):
     return selected_role
 
 
-def assess_ship_design_role(design):
+def assess_ship_design_role(design):  # noqa: max-complexity
     parts = [get_ship_part(partname) for partname in design.parts if partname and get_ship_part(partname)]
 
     if any(p.partClass == fo.shipPartClass.colony and p.capacity == 0 for p in parts):
@@ -450,7 +454,13 @@ def assess_ship_design_role(design):
             return ShipRoleType.BASE_INVASION
 
     if design.speed == 0:
-        if not parts or parts[0].partClass == fo.shipPartClass.shields:  # ToDo: Update logic for new ship designs
+        if not parts or parts[0].partClass in (
+            fo.shipPartClass.shortRange,
+            fo.shipPartClass.fighterBay,
+            fo.shipPartClass.fighterHangar,
+            fo.shipPartClass.shields,
+            fo.shipPartClass.armour,
+        ):
             return ShipRoleType.BASE_DEFENSE
         else:
             return ShipRoleType.INVALID
@@ -459,12 +469,12 @@ def assess_ship_design_role(design):
         return ShipRoleType.MILITARY
     if any(p.partClass == fo.shipPartClass.detection for p in parts):
         return ShipRoleType.CIVILIAN_EXPLORATION
-    else:   # if no suitable role found, use as (bad) scout as it still has inherent detection
+    else:  # if no suitable role found, use as (bad) scout as it still has inherent detection
         warning("Defaulting ship role to 'exploration' for ship with parts: %s", design.parts)
         return ShipRoleType.CIVILIAN_EXPLORATION
 
 
-def generate_fleet_orders_for_fleet_missions():
+def generate_fleet_orders_for_fleet_missions():  # noqa: max-complexity
     """Generates fleet orders from targets."""
     debug("Generating fleet orders")
 
@@ -478,17 +488,19 @@ def generate_fleet_orders_for_fleet_missions():
     debug("Orbital Defense Fleets: %s" % get_empire_fleet_ids_by_role(MissionType.ORBITAL_DEFENSE))
     debug("Outpost Base Fleets: %s" % get_empire_fleet_ids_by_role(MissionType.ORBITAL_OUTPOST))
     debug("Invasion Base Fleets: %s" % get_empire_fleet_ids_by_role(MissionType.ORBITAL_INVASION))
-    debug("Securing Fleets: %s  (currently FLEET_MISSION_MILITARY should be used instead of this Role)" % (
-        get_empire_fleet_ids_by_role(MissionType.SECURE)))
+    debug(
+        "Securing Fleets: %s  (currently FLEET_MISSION_MILITARY should be used instead of this Role)"
+        % (get_empire_fleet_ids_by_role(MissionType.SECURE))
+    )
 
     aistate = get_aistate()
     if fo.currentTurn() < 50:
-        debug('')
+        debug("")
         debug("Explored systems:")
         _print_systems_and_supply(aistate.get_explored_system_ids())
         debug("Unexplored systems:")
         _print_systems_and_supply(aistate.get_unexplored_system_ids())
-        debug('')
+        debug("")
 
     exploration_fleet_missions = aistate.get_fleet_missions_with_any_mission_types([MissionType.EXPLORATION])
     if exploration_fleet_missions:
@@ -514,8 +526,7 @@ def generate_fleet_orders_for_fleet_missions():
     for outpost_fleet_mission in outpost_fleet_missions:
         debug("    %s" % outpost_fleet_mission)
 
-    outpost_base_fleet_missions = aistate.get_fleet_missions_with_any_mission_types(
-        [MissionType.ORBITAL_OUTPOST])
+    outpost_base_fleet_missions = aistate.get_fleet_missions_with_any_mission_types([MissionType.ORBITAL_OUTPOST])
     if outpost_base_fleet_missions:
         debug("Outpost Base targets (must have been interrupted by combat): ")
     else:
@@ -583,7 +594,7 @@ def generate_fleet_orders_for_fleet_missions():
 
 def issue_fleet_orders_for_fleet_missions():
     """Issues fleet orders."""
-    debug('')
+    debug("")
     universe = fo.getUniverse()
     aistate = get_aistate()
     fleet_missions = list(aistate.get_all_fleet_missions())
@@ -598,9 +609,9 @@ def issue_fleet_orders_for_fleet_missions():
             if not fleet or not fleet.shipIDs or fleet_id in universe.destroyedObjectIDs(fo.empireID()):
                 continue
             mission.issue_fleet_orders()
-        fleet_missions = aistate.misc.get('ReassignedFleetMissions', [])
-        aistate.misc['ReassignedFleetMissions'] = []
-    debug('')
+        fleet_missions = aistate.misc.get("ReassignedFleetMissions", [])
+        aistate.misc["ReassignedFleetMissions"] = []
+    debug("")
 
 
 def _print_systems_and_supply(system_ids):
@@ -609,17 +620,18 @@ def _print_systems_and_supply(system_ids):
     fleet_supplyable_system_ids = empire.fleetSupplyableSystemIDs
     for system_id in system_ids:
         system = universe.getSystem(system_id)
-        debug('  %s%s' % (
-            system if system else "  S_%s<>" % system_id,
-            'supplied' if system_id in fleet_supplyable_system_ids else ''))
+        debug(
+            "  %s%s"
+            % (
+                system if system else "  S_%s<>" % system_id,
+                "supplied" if system_id in fleet_supplyable_system_ids else "",
+            )
+        )
 
 
-def get_fighter_capacity_of_fleet(fleet_id):
-    """Return current and max fighter capacity
-
-    :param fleet_id:
-    :type fleet_id: int
-    :return: current and max fighter capacity
+def get_fighter_capacity_of_fleet(fleet_id: int) -> Tuple[int, int]:
+    """
+    Return current and max fighter capacity.
     """
     universe = fo.getUniverse()
     fleet = universe.getFleet(fleet_id)
@@ -637,25 +649,21 @@ def get_fighter_capacity_of_fleet(fleet_id):
     return cur_capacity, max_capacity
 
 
-def get_fuel(fleet_id):
+def get_fuel(fleet_id: int) -> float:
     """Get fuel of fleet.
 
     :param fleet_id: Queried fleet
-    :type fleet_id: int
     :return: fuel of fleet
-    :rtype: float
     """
     fleet = fo.getUniverse().getFleet(fleet_id)
     return fleet and fleet.fuel or 0.0
 
 
-def get_max_fuel(fleet_id):
+def get_max_fuel(fleet_id: int) -> float:
     """Get maximum fuel capacity of fleet.
 
     :param fleet_id: Queried fleet
-    :type fleet_id: int
     :return: max fuel of fleet
-    :rtype: float
     """
     fleet = fo.getUniverse().getFleet(fleet_id)
     return fleet and fleet.maxFuel or 0.0
@@ -671,38 +679,23 @@ def calculate_estimated_time_of_arrival(fleet_id, target_system_id):
     fleet = universe.getFleet(fleet_id)
     if not fleet or not fleet.speed:
         return 99999
-    distance = universe.shortestPathDistance(fleet_id, target_system_id)
+    distance = get_shortest_distance(fleet_id, target_system_id)
     return math.ceil(float(distance) / fleet.speed)
 
 
-def get_fleet_system(fleet):
-    """Return the current fleet location or the target system if currently on starlane.
-
-    :param fleet:
-    :type fleet: UniverseObject.TargetFleet | int
-    :return: current system_id or target system_id if currently on starlane
-    :rtype: int
-    """
+def get_fleet_system(fleet: Union[TargetFleet, int]) -> int:
+    """Return the current fleet location or the target system if currently on starlane."""
     if isinstance(fleet, int):
         fleet = fo.getUniverse().getFleet(fleet)
     return fleet.systemID if fleet.systemID != INVALID_ID else fleet.nextSystemID
 
 
-def get_current_and_max_structure(fleet):
-    """Return a 2-tuple of the sums of structure and maxStructure meters of all ships in the fleet
-
-    :param fleet:
-    :type fleet: int | target.TargetFleet | fo.Fleet
-    :return: tuple of sums of structure and maxStructure meters of all ships in the fleet
-    :rtype: (float, float)
-    """
+def get_current_and_max_structure(fleet: int) -> Tuple[float, float]:
+    """Return a 2-tuple of the sums of structure and maxStructure meters of all ships in the fleet."""
 
     universe = fo.getUniverse()
     destroyed_ids = universe.destroyedObjectIDs(fo.empireID())
-    if isinstance(fleet, int):
-        fleet = universe.getFleet(fleet)
-    elif isinstance(fleet, TargetFleet):
-        fleet = fleet.get_object()
+    fleet = universe.getFleet(fleet)
     if not fleet:
         return (0.0, 0.0)
     ships_cur_health = 0
